@@ -293,9 +293,9 @@ class Sequential():
         else:
             raise ValueError("unknown value for verbose")
     
-    def live_stats(self,live_stats, figure, F1, F2):
+    def live_stats(self,ls_type, figure, F1=0, F2=1, n_row="limited"):
         import matplotlib.pyplot as plt
-        if live_stats == 1:
+        if ls_type == 1:
             figure.clear()
             
             plt.subplot(1,2,1)
@@ -319,34 +319,63 @@ class Sequential():
             figure.canvas.draw()
             figure.canvas.flush_events()
             
-        elif live_stats == 2:
+        elif ls_type == 2:
             figure.clear()
             
             plt.subplot(1,1,1)
             if len(self.train_inputs[0,:]) <= 2 and len(self.train_targets[0,:]) == 1:
-                X_set, y_set = self.train_inputs, self.train_targets
+                if len(self.train_inputs[:,0]) > 1000 and n_row=="limited":
+                    X_set, y_set = self.train_inputs[:1000,:], self.train_targets[:1000,:]
+                elif len(self.train_inputs[:,0]) > 1000 and n_row=="full":
+                    X_set, y_set = self.train_inputs, self.train_targets
+                elif (len(self.train_inputs[:,0]) <= 1000 and n_row=="full") or (len(self.train_inputs[:,0]) <= 1000 and n_row=="limited"):
+                    X_set, y_set = self.train_inputs, self.train_targets
+                else:
+                    raise ValueError("invalid value for 'n_row'")
+                    
                 X1, X2 = np.meshgrid(np.arange(start = X_set[:, F1].min() - 1, stop = X_set[:, F1].max() + 1, step = 0.01),
                                      np.arange(start = X_set[:, F2].min() - 1, stop = X_set[:, F2].max() + 1, step = 0.01))
                 plt.contourf(X1, X2, self.predict(np.array([X1.ravel(), X2.ravel()]).T).reshape(X1.shape),
                              alpha = 0.3, cmap = "coolwarm")
-                
-            plt.scatter(self.train_inputs[:,F1], self.train_inputs[:,F2], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
             
+            if len(self.train_inputs[:,0]) > 1000 and n_row=="limited":
+                plt.scatter(self.train_inputs[:1000,F1], self.train_inputs[:1000,F2], c=self.train_outputs[:1000,:], cmap="coolwarm", alpha=1)
+            elif len(self.train_inputs[:,0]) > 1000 and n_row=="full":
+                plt.scatter(self.train_inputs[:,F1], self.train_inputs[:,F2], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
+            elif (len(self.train_inputs[:,0]) <= 1000 and n_row=="full") or (len(self.train_inputs[:,0]) <= 1000 and n_row=="limited"):
+                plt.scatter(self.train_inputs[:,F1], self.train_inputs[:,F2], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
+            else:
+                raise ValueError("invalid value for 'n_row'")
+                
             figure.canvas.draw()
             figure.canvas.flush_events()
             
-        elif live_stats == -1:
+        elif ls_type == -1:
             figure.clear()
             
             plt.subplot(1, 2, 2)
             if len(self.train_inputs[0,:]) <= 2 and len(self.train_targets[0,:]) == 1:
-                X_set, y_set = self.train_inputs, self.train_targets
+                if len(self.train_inputs[:,0]) > 1000 and n_row=="limited":
+                    X_set, y_set = self.train_inputs[:1000,:], self.train_targets[:1000,:]
+                elif len(self.train_inputs[:,0]) > 1000 and n_row=="full":
+                    X_set, y_set = self.train_inputs, self.train_targets
+                elif (len(self.train_inputs[:,0]) <= 1000 and n_row=="full") or (len(self.train_inputs[:,0]) <= 1000 and n_row=="limited"):
+                    X_set, y_set = self.train_inputs, self.train_targets
+                else:
+                    raise ValueError("invalid value for 'n_row'")
                 X1, X2 = np.meshgrid(np.arange(start = X_set[:, F1].min() - 1, stop = X_set[:, F1].max() + 1, step = 0.01),
                                      np.arange(start = X_set[:, F2].min() - 1, stop = X_set[:, F2].max() + 1, step = 0.01))
                 plt.contourf(X1, X2, self.predict(np.array([X1.ravel(), X2.ravel()]).T).reshape(X1.shape),
                              alpha = 0.3, cmap = "coolwarm")
                 
-            plt.scatter(self.train_inputs[:,F1], self.train_inputs[:,F2], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
+            if len(self.train_inputs[:,0]) > 1000 and n_row=="limited":
+                plt.scatter(self.train_inputs[:1000,F1], self.train_inputs[:1000,F2], c=self.train_outputs[:1000,:], cmap="coolwarm", alpha=1)
+            elif len(self.train_inputs[:,0]) > 1000 and n_row=="full":
+                plt.scatter(self.train_inputs[:,F1], self.train_inputs[:,F2], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
+            elif (len(self.train_inputs[:,0]) <= 1000 and n_row=="full") or (len(self.train_inputs[:,0]) <= 1000 and n_row=="limited"):
+                plt.scatter(self.train_inputs[:,F1], self.train_inputs[:,F2], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
+            else:
+                raise ValueError("invalid value for 'n_row'")
             
             plt.subplot(2,2,1)
             plt.title("Loss Evolution")
@@ -368,12 +397,22 @@ class Sequential():
             figure.canvas.draw()
             figure.canvas.flush_events()
     
-    def fit(self, cost_fn, epochs = 1000, lr = 0.1, momentum = None, verbose = 1, live_stats = 0, F1=0, F2=1):
+    def fit(self, cost_fn, epochs = 1000, lr = 0.1, momentum = None, verbose = 1, live_stats = {"type": 0, "F1": 0, "F2": 1, "n_row": "limited"}):
         for n in [-1,0,1,2]:
-            if live_stats == n:
-                break
-            elif n == 2 and live_stats != n:
-                raise ValueError("unknown value for live_stats")
+            if type(live_stats) == dict:
+                if live_stats["type"] == n:
+                    break
+                elif n == 2 and live_stats["type"] != n:
+                    raise ValueError("unknown value for live_stats['type']")
+            elif type(live_stats) == int:
+                if live_stats == n:
+                    break
+                elif n == 2 and live_stats != n:
+                    raise ValueError("unknown value for live_stats")
+                    
+        if type(live_stats) == dict:
+            if live_stats["n_row"] != "full" and live_stats["n_row"] != "limited":
+                raise ValueError("unknown value for live_stats['n_row']")
                 
         self.__is_trained__ = True
         import copy
@@ -394,13 +433,17 @@ class Sequential():
                 if cost_fn == name:
                     self.val_cost_fn = function
                     break
-                
-        if live_stats == -1:
+        
+        if type(live_stats) == int:
+            live_stats = {"type": live_stats, "F1": 0, "F2": 1, "n_row": "limited"}
+        
+        if live_stats["type"] == -1:
             update = 50
             figure = plt.figure()
-        elif live_stats == 0:
+        elif live_stats["type"] == 0:
             update = 1
-        elif live_stats == 1 or live_stats == 2:
+            figure = None
+        elif live_stats["type"] == 1 or live_stats["type"] == 2:
             update = 25
             figure = plt.figure()
             
@@ -418,7 +461,7 @@ class Sequential():
                 
                 if live_stats != 0:
                     if epoch % update == 0:
-                        self.live_stats(live_stats, figure, F1, F2)
+                        self.live_stats(ls_type = live_stats["type"], figure = figure, F1 = live_stats["F1"], F2 = live_stats["F2"], n_row = live_stats["n_row"])
                             
                 self.verbose(verbose, epoch, epochs, start)
         else:
@@ -435,7 +478,7 @@ class Sequential():
                 
                 if live_stats != 0:
                     if epoch % update == 0:
-                        self.live_stats(live_stats, figure, F1, F2)
+                        self.live_stats(ls_type = live_stats["type"], figure = figure, F1 = live_stats["F1"], F2 = live_stats["F2"], n_row = live_stats["n_row"])
                         
                 self.verbose(verbose, epoch, epochs, start)
         
